package no.malmanger.seraph;

import com.atlassian.confluence.user.ConfluenceAuthenticator;
import com.atlassian.confluence.user.UserAccessor;
import com.atlassian.seraph.auth.AuthenticatorException;
import com.atlassian.spring.container.ContainerManager;
import com.atlassian.user.Group;
import com.atlassian.user.GroupManager;
import com.atlassian.user.User;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.security.Principal;
import java.util.Iterator;

public class RemoteUserAuthenticator extends ConfluenceAuthenticator {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7552238508680156644L;
    private static final Logger log = Logger.getLogger(RemoteUserAuthenticator.class);
	
    private static RemoteUserConfiguration config;
    
    private GroupManager groupManager = null;

    static {
//    	log.setLevel(Level.DEBUG);
    	config = RemoteUserConfigLoader.getRemoteUserConfiguration(null);
    }

    public Principal getUser(HttpServletRequest request, HttpServletResponse response) {
    	log.debug("getUser(request, response)" + request.toString());
        Principal user = null;
        
        try {
            if(request.getSession() != null && request.getSession().getAttribute(ConfluenceAuthenticator.LOGGED_IN_KEY) != null) {
                log.debug("Session found; user already logged in");
                user = (Principal) request.getSession().getAttribute(ConfluenceAuthenticator.LOGGED_IN_KEY);
            } else {

            	String remoteUserHeaderName = config.getRemoteUserHeaderName();

            	String username = request.getHeader(remoteUserHeaderName);
                log.debug("NOW remoteUser="+username);         
                
                if (username != null) {
                	username = username.toLowerCase();
                	user = getUser(username);
                	
                	if (user == null) {
                		// try creating
                		log.debug("creating new user:" + username);
                		user = createUser(username);
                		if (user != null) {
                			addDefaultGroups((User) user);
                		}
                	}
                	                	
                	updateUser(user, request.getHeader(config.getFullNameHeaderName()), request.getHeader(config.getEmailHeaderName()));
                	if (config.isUpdateGroups()) {
                		log.debug("Update default groups");
                		addDefaultGroups((User) user); 
                	}
                	log.debug("Logged in via SSO, with User " + user);
                	request.getSession().setAttribute(ConfluenceAuthenticator.LOGGED_IN_KEY, user);
                	request.getSession().setAttribute(ConfluenceAuthenticator.LOGGED_OUT_KEY, null);
                	
                }
            }
        } catch (Exception e) { // catch class cast exceptions
            log.warn("Exception: " + e, e);
        }
        
       	return user;
    }
    
    public Principal getUser(String username) {
    	log.debug("getUser(String");
    	Principal user = super.getUser(username);
    	return user;
    }
    
    public Principal getUserFromBasicAuthentication(HttpServletRequest request, HttpServletResponse response) {
    	log.debug("getUserFromBasicAuthentication" + request.toString());
    	return super.getUserFromBasicAuthentication(request, response);
    }
    
    public Principal getUserFromSession(HttpServletRequest request) {
    	log.debug("getUserFromSession" + request.toString());
    	return super.getUserFromSession(request);
    }

    public Principal getUserFromCookie(HttpServletRequest request, HttpServletResponse response) {
    	log.debug("getUserFromCookie" + request.toString());
    	return super.getUserFromCookie(request, response);
    }

    public boolean authenticate(Principal user, String password) {
    	log.debug("authenticate");
    	return super.authenticate(user, password);
    }

    public boolean login(HttpServletRequest request, HttpServletResponse response, String username, String password, boolean b) throws AuthenticatorException {
    	log.debug("getUserFromBasicAuthentication" + request.toString());
    	return super.login(request, response, username, password, b);
    }
    
    private Principal createUser(String username) {
        UserAccessor userAccessor = getUserAccessor();
        Principal user = null;

        if (config.isCreateUsers()) {
            if (log.isInfoEnabled()) {
                log.info("Creating user account for " + username);
            }

            try {
                user = userAccessor.createUser(username);
                
                // add to default groups
                addDefaultGroups((User)user);
                
            } catch (Throwable t) {
                // Note: just catching EntityException like we used to do didn't
                // seem to cover Confluence massive with Oracle
                log.debug("Error creating user " + username + ". Assume it's created already", t);
                user = getUser(username);

                if (user == null) { // User still don't exist
                    log.error("Error creating user " + username + ". Got null user after attempted to create user (so it probably was not a duplicate).", t);
                }
            }
        } else {
            log.debug("Cannot create new user. Please use " + RemoteUserConfiguration.CREATE_USERS + "=true to enable this feature. Tried to create username=" + username);
        }

        return user;
    }

    private void addDefaultGroups(User user) {
    	Iterator<String> iterator = config.getDefaultGroups().iterator();
    	String groupname;
    	Group group;
    	
    	while (iterator.hasNext()) {
    		groupname = (String) iterator.next();
    		if (groupname.length() == 0) { continue; } // Skip if emty string
    		log.debug(" user to group " + groupname);
    		try {
	    		group = getGroupManager().getGroup(groupname);
	    		if (group != null) {
	    			if (! getGroupManager().hasMembership(group, user)) {
	    				getGroupManager().addMembership(group, user);
	    			}
	    		}
    		} catch (Exception e) {
    			log.error("Unable to add user " + user.getName() + " to group " + groupname + " - " + e.getMessage(), e);
    		}
    	}
    	
    	for (String element: config.getDefaultGroups()) {
    		log.debug("      Group element = " + element);
    	}
    }
    
    private GroupManager getGroupManager() {
    	if (this.groupManager == null) { groupManager = (GroupManager) ContainerManager.getComponent("groupManager"); }
    	return this.groupManager;
    }
    
    private void updateUser(Principal user, String fullName, String emailAddress) {
        UserAccessor userAccessor = getUserAccessor();

        // If we have new values for name or email, update the user object
        if ((user != null) && (user instanceof User)) {
            User userToUpdate = (User) user;
            boolean updated = false;

            if (userAccessor.isReadOnly(userToUpdate)) {
                if (log.isDebugEnabled()) {
                    log.debug("not updating user, because user is read-only");
                }
                return;
            }
            
            if ((fullName != null) && !fullName.equals(userToUpdate.getFullName())) {
                if (log.isDebugEnabled()) {
                    log.debug("updating user fullName to '" + fullName + "'");
                }

                userToUpdate.setFullName(fullName);
                updated = true;
            } else {
                if (log.isDebugEnabled()) {
                    log.debug(
                        "new user fullName is same as old one: '" + fullName + "'");
                }
            }

            if ((emailAddress != null) && !emailAddress.equals(userToUpdate.getEmail())) {
                if (log.isDebugEnabled()) {
                    log.debug(
                        "updating user emailAddress to '" + emailAddress + "'");
                }
                
                userToUpdate.setEmail(emailAddress);
                updated = true;
            } else {
                if (log.isDebugEnabled()) {
                    log.debug(
                        "new user emailAddress is same as old one: '" + emailAddress + "'");
                }
            }

            if (updated) {
                try {
                    userAccessor.saveUser(userToUpdate);
                } catch (Throwable t) {
                    log.error("Couldn't update user " + userToUpdate.getName(),
                        t);
                }
            }
        }
    }
}

